package ical

import (
	"bytes"
	"encoding/json"
	"htwkalender/model"
	"htwkalender/service/db"
	"time"

	"github.com/jordic/goics"
	"github.com/pocketbase/pocketbase"
	"github.com/pocketbase/pocketbase/apis"
)

const expirationTime = 5 * time.Minute

func Feed(app *pocketbase.PocketBase, token string) (string, error) {
	var result string

	feed, err := db.FindFeedByToken(token, app)
	if feed == nil && err != nil {
		return "", err
	}

	modules := make(map[string]model.FeedCollection)
	var modulesArray []model.FeedCollection
	_ = json.Unmarshal([]byte(feed.Modules), &modulesArray)

	for _, module := range modulesArray {
		modules[module.UUID] = module
	}

	newFeed, err := createFeedForToken(app, modules)
	if err != nil {
		return "", err
	}
	result = newFeed.Content

	return result, nil
}

func createFeedForToken(app *pocketbase.PocketBase, modules map[string]model.FeedCollection) (*model.FeedModel, error) {
	res, err := db.GetPlanForModules(app, modules)

	if err != nil {
		return nil, apis.NewNotFoundError("Could not fetch events", err)
	}

	b := bytes.Buffer{}
	goics.NewICalEncode(&b).Encode(IcalModel{Events: res, Mapping: modules})
	feed := &model.FeedModel{Content: b.String(), ExpiresAt: time.Now().Add(expirationTime)}
	return feed, nil
}

func CreateIndividualFeed(requestBody []byte, app *pocketbase.PocketBase) (string, error) {
	var modules []model.FeedCollection

	err := json.Unmarshal(requestBody, &modules)
	if err != nil {
		return "", apis.NewNotFoundError("Could not parse request body", err)
	}

	var feed model.Feed
	jsonModules, _ := json.Marshal(modules)
	feed.Modules = string(jsonModules)

	collection, dbError := db.FindCollection(app, "feeds")
	if dbError != nil {
		return "", apis.NewNotFoundError("Collection could not be found", dbError)
	}

	record, err := db.SaveFeed(feed, collection, app)
	if err != nil {
		return "", apis.NewNotFoundError("Could not save feed", err)
	}

	return record.Id, nil
}
